@@ -96,7 +96,7 @@ typedef struct {
 	GtkWidget         *info_bar_vbox;
 	GtkWidget         *search_bar;
 
-	GSList            *pending_messages;
+	guint              unread_messages;
 	/* TRUE if the pending messages can be displayed. This is to avoid to show
 	 * pending messages *before* messages from logs. (#603980) */
 	gboolean           can_show_pending;
@@ -1116,9 +1116,7 @@ chat_message_received (EmpathyChat *chat, EmpathyMessage *message)
 			       TP_CHANNEL_CHAT_STATE_ACTIVE,
 			       chat);
 
-	priv->pending_messages = g_slist_prepend (priv->pending_messages,
-			g_object_ref (message));
-
+	priv->unread_messages++;
 	g_signal_emit (chat, signals[NEW_MESSAGE], 0, message);
 }
 
@@ -2265,7 +2263,7 @@ chat_finalize (GObject *object)
 			chat_members_changed_cb, chat);
 		g_signal_handlers_disconnect_by_func (priv->tp_chat,
 			chat_remote_contact_changed_cb, chat);
-		empathy_tp_chat_close (priv->tp_chat);
+		empathy_tp_chat_leave (priv->tp_chat);
 		g_object_unref (priv->tp_chat);
 	}
 	if (priv->account) {
@@ -2934,9 +2932,9 @@ empathy_chat_get_nb_unread_messages (EmpathyChat *self)
 {
 	EmpathyChatPriv *priv = GET_PRIV (self);
 
-	g_return_val_if_fail (EMPATHY_IS_CHAT (self), FALSE);
+	g_return_val_if_fail (EMPATHY_IS_CHAT (self), 0);
 
-	return g_slist_length (priv->pending_messages);
+	return priv->unread_messages;
 }
 
 /* called when the messages have been read by user */
@@ -2947,12 +2945,9 @@ empathy_chat_messages_read (EmpathyChat *self)
 
 	g_return_if_fail (EMPATHY_IS_CHAT (self));
 
-	empathy_tp_chat_acknowledge_messages (priv->tp_chat,
-			priv->pending_messages);
-
-	while (priv->pending_messages != NULL) {
-		g_object_unref (priv->pending_messages->data);
-		priv->pending_messages = g_slist_delete_link (
-			priv->pending_messages, priv->pending_messages);
+	if (priv->tp_chat != NULL) {
+		empathy_tp_chat_acknowledge_all_messages (priv->tp_chat);
 	}
+	
+	priv->unread_messages = 0;
 }
