@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2003-2007 Imendio AB
  * Copyright (C) 2007-2008 Collabora Ltd.
@@ -32,6 +31,8 @@
 #include <glib/gi18n-lib.h>
 
 #include <libxml/uri.h>
+
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/connection.h>
 #include <telepathy-glib/channel.h>
 #include <telepathy-glib/dbus.h>
@@ -135,7 +136,8 @@ gboolean
 empathy_xml_validate (xmlDoc      *doc,
 		     const gchar *dtd_filename)
 {
-	gchar        *path, *escaped;
+	gchar        *path;
+	xmlChar      *escaped;
 	xmlValidCtxt  cvp;
 	xmlDtd       *dtd;
 	gboolean      ret;
@@ -149,7 +151,8 @@ empathy_xml_validate (xmlDoc      *doc,
 	DEBUG ("Loading dtd file %s", path);
 
 	/* The list of valid chars is taken from libxml. */
-	escaped = xmlURIEscapeStr (path, ":@&=+$,/?;");
+	escaped = xmlURIEscapeStr ((const xmlChar *) path,
+		(const xmlChar *)":@&=+$,/?;");
 	g_free (path);
 
 	memset (&cvp, 0, sizeof (cvp));
@@ -172,7 +175,7 @@ empathy_xml_node_get_child (xmlNodePtr   node,
         g_return_val_if_fail (child_name != NULL, NULL);
 
 	for (l = node->children; l; l = l->next) {
-		if (l->name && strcmp (l->name, child_name) == 0) {
+		if (l->name && strcmp ((const gchar *) l->name, child_name) == 0) {
 			return l;
 		}
 	}
@@ -212,12 +215,12 @@ empathy_xml_node_find_child_prop_value (xmlNodePtr   node,
 	for (l = node->children; l && !found; l = l->next) {
 		xmlChar *prop;
 
-		if (!xmlHasProp (l, prop_name)) {
+		if (!xmlHasProp (l, (const xmlChar *) prop_name)) {
 			continue;
 		}
 
-		prop = xmlGetProp (l, prop_name);
-		if (prop && strcmp (prop, prop_value) == 0) {
+		prop = xmlGetProp (l, (const xmlChar *) prop_name);
+		if (prop && strcmp ((const gchar *) prop, prop_value) == 0) {
 			found = l;
 		}
 
@@ -239,7 +242,7 @@ empathy_presence_get_default_message (TpConnectionPresenceType presence)
 	case TP_CONNECTION_PRESENCE_TYPE_EXTENDED_AWAY:
 		return _("Away");
 	case TP_CONNECTION_PRESENCE_TYPE_HIDDEN:
-		return _("Hidden");
+		return _("Invisible");
 	case TP_CONNECTION_PRESENCE_TYPE_OFFLINE:
 		return _("Offline");
 	case TP_CONNECTION_PRESENCE_TYPE_UNSET:
@@ -275,6 +278,43 @@ empathy_presence_from_str (const gchar *str)
 	return TP_CONNECTION_PRESENCE_TYPE_UNSET;
 }
 
+const gchar *
+empathy_status_reason_get_default_message (TpConnectionStatusReason reason)
+{
+	switch (reason) {
+	case TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED:
+		return _("No reason specified");
+	case TP_CONNECTION_STATUS_REASON_REQUESTED:
+		return _("Status is set to offline");
+	case TP_CONNECTION_STATUS_REASON_NETWORK_ERROR:
+		return _("Network error");
+	case TP_CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED:
+		return _("Authentication failed");
+	case TP_CONNECTION_STATUS_REASON_ENCRYPTION_ERROR:
+		return _("Encryption error");
+	case TP_CONNECTION_STATUS_REASON_NAME_IN_USE:
+		return _("Name in use");
+	case TP_CONNECTION_STATUS_REASON_CERT_NOT_PROVIDED:
+		return _("Certificate not provided");
+	case TP_CONNECTION_STATUS_REASON_CERT_UNTRUSTED:
+		return _("Certificate untrusted");
+	case TP_CONNECTION_STATUS_REASON_CERT_EXPIRED:
+		return _("Certificate expired");
+	case TP_CONNECTION_STATUS_REASON_CERT_NOT_ACTIVATED:
+		return _("Certificate not activated");
+	case TP_CONNECTION_STATUS_REASON_CERT_HOSTNAME_MISMATCH:
+		return _("Certificate hostname mismatch");
+	case TP_CONNECTION_STATUS_REASON_CERT_FINGERPRINT_MISMATCH:
+		return _("Certificate fingerprint mismatch");
+	case TP_CONNECTION_STATUS_REASON_CERT_SELF_SIGNED:
+		return _("Certificate self-signed");
+	case TP_CONNECTION_STATUS_REASON_CERT_OTHER_ERROR:
+		return _("Certificate error");
+	default:
+		return _("Unknown reason");
+	}
+}
+
 gchar *
 empathy_file_lookup (const gchar *filename, const gchar *subdir)
 {
@@ -351,6 +391,15 @@ empathy_uint_compare (gconstpointer a,
 gchar *
 empathy_protocol_icon_name (const gchar *protocol)
 {
+  if (!tp_strdiff (protocol, "yahoojp"))
+    /* Yahoo Japan uses the same icon as Yahoo */
+    protocol = "yahoo";
+  else if (!tp_strdiff (protocol, "simple"))
+    /* SIMPLE uses the same icon as SIP */
+    protocol = "sip";
+  else if (!tp_strdiff (protocol, "sms"))
+    return g_strdup ("phone");
+
   return g_strdup_printf ("im-%s", protocol);
 }
 
@@ -488,3 +537,72 @@ empathy_signal_connect_weak (gpointer instance,
   g_object_weak_ref (instance_obj, instance_destroyed_cb, ctx);
   g_object_weak_ref (user_data, user_data_destroyed_cb, ctx);
 }
+
+/* Note: this function depends on the account manager having its core feature
+ * prepared. */
+TpAccount *
+empathy_get_account_for_connection (TpConnection *connection)
+{
+  TpAccountManager *manager;
+  TpAccount *account = NULL;
+  GList *accounts, *l;
+
+  manager = tp_account_manager_dup ();
+
+  accounts = tp_account_manager_get_valid_accounts (manager);
+
+  for (l = accounts; l != NULL; l = l->next)
+    {
+      TpAccount *a = l->data;
+
+      if (tp_account_get_connection (a) == connection)
+        {
+          account = a;
+          break;
+        }
+    }
+
+  g_list_free (accounts);
+  g_object_unref (manager);
+
+  return account;
+}
+
+gboolean
+empathy_account_manager_get_accounts_connected (gboolean *connecting)
+{
+  TpAccountManager *manager;
+  GList *accounts, *l;
+  gboolean out_connecting = FALSE;
+  gboolean out_connected = FALSE;
+
+  manager = tp_account_manager_dup ();
+
+  if (G_UNLIKELY (!tp_account_manager_is_prepared (manager,
+          TP_ACCOUNT_MANAGER_FEATURE_CORE)))
+    g_critical (G_STRLOC ": %s called before AccountManager ready", G_STRFUNC);
+
+  accounts = tp_account_manager_get_valid_accounts (manager);
+
+  for (l = accounts; l != NULL; l = l->next)
+    {
+      TpConnectionStatus s = tp_account_get_connection_status (
+          TP_ACCOUNT (l->data), NULL);
+
+      if (s == TP_CONNECTION_STATUS_CONNECTING)
+        out_connecting = TRUE;
+      else if (s == TP_CONNECTION_STATUS_CONNECTED)
+        out_connected = TRUE;
+
+      if (out_connecting && out_connected)
+        break;
+    }
+
+  g_list_free (accounts);
+  g_object_unref (manager);
+
+  if (connecting != NULL)
+    *connecting = out_connecting;
+
+  return out_connected;
+}
